function [P] = basis_emp(K,ext,mom)
%% BASIS_EMP Polynomial chaos basis constrauction using moment substitution.
%   [P] = BASIS_EMP(K,s,ext) return basis polynomials (K+1 per dimension,
%   ND=length(mom(1,:)))
%
%   Require 2+K-1 moments per dimension (2+K if ext==norm)
%
%   Default values for ext is '' (no normalization).
%   Warning: might require a second orthogonalization for more precision

if length(mom(1,:))<2+K || (length(mom(1,:))<2+K-1 && strcmp(ext,'norm'))
    disp(['ERROR (basis_emp): Require ',num2str(2+K-1),' moments per dimension (',num2str(2+K),' if norm.), given ',num2str(length(mom(1,:))),').']); 
    return;
end

ND=length(mom(:,1));
if ~exist('ext','var'), ext=''; end
xi = sym('xi',[1 ND]);
assumeAlso(xi,'real');
P=sym('P',[K+1 ND]);
P(1,:) = ones(1,ND);
P(2,:) = xi;

for d=1:ND
    A = (xi(d)*ones(2*K+1,1)).^([0:2*K]'); 
    P(:,d) = gs_ortho_emp(A(1:K+1),A,mom(d,:));
    P(:,d) = simplify(P(:,d));
    if strcmp(ext,'norm')
        P(:,d) = gs_norm_emp(P(:,d),A,mom(d,:));
    end
end

assumeAlso(P,'real')
end
