function P = gs_ortho_emp(P,A,mom)
%% ORTHOGONALIZATION of polynomials.
%   P = gs_ortho_emp(P,A,mom) orthogonalization of polynomials in P by
%   Gram-Schmidt procedure. Computing the inner product by replacing
%   monomials in A with the given moments instead of computing the integrals.

digits(32)
TEPS=10e-30;

disp('Gram-Schmidt ORTHOGONALIZATION (moment mode)')
for j = 1:length(P)
    x = P(j);
    for i = 1:j-1
        norm=expand(P(i)*P(i));
        fac=expand(x*P(i));
        for m=length(A):-1:2
            norm=subs(norm,A(m),mom(m));
            fac=subs(fac,A(m),mom(m));
        end
        if abs(vpa(norm))<TEPS, warning(['SMALL integral (gs_ortho_emp): ',char(norm)]);
        elseif norm<=0, error(['NEGATIVE integral (gs_ortho_emp): ',char(norm)]); 
        else temp=fac/norm;
        end
        x = x - temp*P(i);
    end
    P(j)=expand(x);
end

P=expand(P);