function F = pce_setup(name,f,xd,xdd,xa,p,NL,I,s,ext,b)
%% GALERKIN projection
%   F = pce_setup(name,f,xd,xdd,xa,p,NL,K,s,ext,a,b) sets-up environment
%   for polynomial chaos projection.

global NOGALERKIN LIN EXPL TENSORPATH FILEPATH
TENSORPATH='ISPoc/PCE/TENSOR/';
FILEPATH='ISPoc/problemfiles/TXT/';
NOGALERKIN=0; LIN=0; EXPL=0;
format shortG

%% initial set-up
K=max(I(:,1));
TK=length(I(:,1));
%for i=1:length(s), [mom(i,:),momnorm(i,:)] = moments((NL+1)*K(k)+1,s(i)); end
[PHI,w,b] = basis(K,s,ext,b);
if (s(1)==1 && b(1,1)==-inf && b(1,2)==inf) || (s(1)>1 && b(1,1)==-1 && b(1,2)==1), str='';
else str=['[',num2str(b(1,1)),',',num2str(b(1,2)),']'];
end

NXD = length(xd);
NXA = length(xa);
F = sym('F',[1 NXD*K]);
if isempty(f), return; end

% build PCE variables
[XD,XDD,XA,Xd,Xdd,Xa,Xdc,Xddc,Xac,P,Phi] = defPCE(I,NXD,NXA,length(p));

tname = ['K',num2str(K),'_ND',num2str(length(s)),'s',num2str(s(1)),ext,str];
fname = [FILEPATH,name,'_',tname,'.txt'];
fid = fopen(fname,'a');
fprintf(fid,'\n%s %s\nUncertain variables p=%s replaced by P=%s\n',date,name,char(p),char(P));
fprintf(fid,'Basis polynomial indices I=%s\n',char(vpa(I,2)));
fprintf(fid,'\nOriginal equations\n');
for i=1:length(f)
    S=['res[',num2str(i-1),'] = ',char(f(i)),';'];
    fprintf(fid,'%s\n',S);
    %disp(['Number of terms ',num2str(length(children(expand(f(i)))))])
end

T =  tensor(tname,NL+1,I,PHI,w,b);


%% PCE transformation
tic
% substitute expansions into equations
fpce = subs(f,[p, xd, xdd, xa],[P, Xd, Xdd, Xa]);
% call transformation algorithm
F=PCEtransform(fpce,Xdc,Xddc,XD,XDD,Xac,XA,Phi,PHI,I,T);

%% Print transformed equations to file
assumeAlso(symvar(F),'real')
if EXPL
    fprintf(fid,'\nTransformed explicit system\n');
    if length(xdd)==length(f)
        F=solveExplicit(F,XDD,fid);
    elseif length(xa)==length(f)
        F=solveExplicit(F,XA,fid);
    end
    if LIN
        solveLinear(F,XA,fid);
    end
else
    fprintf(fid,'\nTransformed implicit system\n');
    printEquations(F,fid);
end
disp(['TIME for Galerkin projection is ',num2str(toc),' s. Save output in ',fname,'.']);
fclose(fid);

%% Transform output to MUSCOD (C-)style
%system(['./format.sh ',fname]);  %open(fname);

end


function H = PCEtransform(h,Xdc,Xddc,XD,XDD,Xac,XA,Phi,PHI,I,T)
%% PCE Transformation
%   H = PCEtransform(h,Xdc,Xddc,XD,XDD,Xac,XA,Phi,PHI,I,T) calls Galerkin 
%   projection for each equation & orders resulting system. 

MK=length(I(:,1));
xi=symvar(PHI);

GALERKIN = 1;
PRINTOUT = 0;

NXD = length(XD)/MK;
NXA = length(XA)/MK;

if GALERKIN
    E = sym('E',[length(h),MK]);
    for i=1:length(h)  % parfor possible
        temp = galerkin(h(i),Phi,I,T); % temp contains K equations
        E(i,:) = temp;
        if PRINTOUT; fprintf('\n'); %#ok<PFGV>
            fprintf('Galerkin-PCE Method for equation %d: %s\n',i,char(h(i)));
            fprintf('Transformed equations: %s\n',char(E(i,:)));
        end
    end
else
    B = sym('B',[MK 1]);
    B=subs(B,B,ones(MK,1));
    E = sym('E',[length(h),2]);
    for i=1:length(h)
        % write equation i.t.o. Phi and i.t.o. xi
        for d=1:length(I(1,:)) % build MD polynomials of MV-order 1,..,K
            B=B.*PHI(I(:,d)+1,d);
        end
        temp=collect(subs(h(i),Phi,B'),xi);
        E(i,:) = [h(i),temp];
    end
end

for i=1:length(h)
    for j=1:length(E(1,:))
        for l=1:MK
            E(i,j) = subs(E(i,j),[Xdc(:,l)', Xddc(:,l)'],[XD((l-1)*NXD+1:l*NXD), XDD((l-1)*NXD+1:l*NXD)]);
            E(i,j) = subs(E(i,j),Xac(:,l)',XA((l-1)*NXA+1:l*NXA));
        end
    end
end
% reorder equations: first states XD1..XDNXD nominal states (0-order PCE coefficients), following states 1-order PCE coefficients,
H = sym('H',[1 length(h)*length(E(1,:))]);  % new (implicit) differential equations
n=1;
for j=1:length(E(1,:))
    for i=1:length(h)
        H(n) = simplify(E(i,j)); % E(i,j)=H((i-1)*K+j)="jth coeff of ith equ"
        n = n+1;
    end
end


end


function [XD,XDD,XA, Xd, Xdd, Xa, Xdc, Xddc, Xac, P, Phi] = defPCE(I,NXD,NXA,NP)
%% Define involved quantities
%   [XD,XDD,XA, Xd, Xdd, Xa, Xdc, Xddc, Xac, P, Phi] = defPCE(K,NXD,NXA,NQ)
%   Constructs PC expansion of involved variables (differential & algebraic
%   variable, parameters, abstract basis functions)

% initialize uncertain parameters with mean and std.dev
INIT_PC=1;

TK=length(I(:,1));

% new variables (sorted PCE coefficients)
XD = sym('XD',[1 NXD*TK]);
XDD = sym('XDD',[1 NXD*TK]);
XA = sym('XA',[1 NXA*TK]);

% coefficients of influenced variables (unsorted PCE coefficients)
Xdc = sym('Xd_',[NXD TK]);
Xddc = sym('Xdd_',[NXD TK]);
Xac = sym('Xa_',[NXA TK]);
assumeAlso(Xdc,'real');
assumeAlso(Xddc,'real');
assumeAlso(Xac,'real');

% coefficients of uncertain parameters (initialized)
PC = sym('p%d%d',[NP TK]);
PC = subs(PC,PC,zeros(NP,TK));
if INIT_PC
    PC(:,1)=sym('mu',[1 NP]);
    for i=1:TK
        J=I(i,:);
        if isequal(J,[1]), PC(i)=sym('sigma'); end  
        if isequal(J,[1,0]), PC(1,i)=sym('sigma1'); end
        if isequal(J,[0,1]), PC(2,i)=sym('sigma2'); end  
        if isequal(J,[1,0,0]), PC(1,i)=sym('sigma1'); end
        if isequal(J,[0,1,0]), PC(2,i)=sym('sigma2'); end   
        if isequal(J,[0,0,1]), PC(3,i)=sym('sigma3'); end
        if isequal(J,[1,0,0,0]), PC(1,i)=sym('sigma1'); end
        if isequal(J,[0,1,0,0]), PC(2,i)=sym('sigma2'); end   
        if isequal(J,[0,0,1,0]), PC(3,i)=sym('sigma3'); end
        if isequal(J,[0,0,0,1]), PC(4,i)=sym('sigma4'); end
        if isequal(J,[1,0,0,0,0]), PC(1,i)=sym('sigma1'); end
        if isequal(J,[0,1,0,0,0]), PC(2,i)=sym('sigma2'); end   
        if isequal(J,[0,0,1,0,0]), PC(3,i)=sym('sigma3'); end
        if isequal(J,[0,0,0,1,0]), PC(4,i)=sym('sigma4'); end   
        if isequal(J,[0,0,0,0,1]), PC(5,i)=sym('sigma5'); end
    end
end
assumeAlso(PC,'real');

% PC expansions
Phi = sym('Phi',[1 TK]); % MD basis polynomials
assumeAlso(Phi,'real');
P = sym('P',[1 NP]);
Xd = sym('Xd',[1 NXD]);
Xdd = sym('Xdd',[1 NXD]);
Xa = sym('Xa',[1 NXA]);
for i=1:NP % replace each uc parameter with finite expansion
    P(i) = dot(PC(i,:),Phi);
end

for i=1:NXD % replace each state variable with truncated expansion
    Xd(i) = dot(Xdc(i,:),Phi);
    Xdd(i) = dot(Xddc(i,:),Phi);
end
for i=1:NXA % replace each state variable with truncated expansion
    Xa(i) = dot(Xac(i,:),Phi);
end

end
%-------------------------------------------------------------------------%


function [] = printEquations(F,fid)

for i=1:length(F) % equations per coefficient
    %S=['res[',num2str(i-1),'] = ',char(F(i)),';'];
    S=['res[',num2str(i-1),'] = ',char(vpa(F(i),16)),';'];
    fprintf(fid,'%s\n',S);
end
end
%-------------------------------------------------------------------------%

% compute (explicit) expression (X=G(Z))
function H = solveExplicit(F,X,fid)
N=length(F);
tic
cellF = num2cell(F==0);
cellX = num2cell(X);
H = sym('H',[1 N]);
s=solve(cellF{1,:},cellX{1,:});
s=struct2cell(s);
for i=1:N
    H(i)=simplify(s{i});
end
disp(['Time for explicit solve is ',num2str(toc),'s.'])
printEquations(H,fid);

end
%-------------------------------------------------------------------------%

% compute (explicit) expression (X=G(Z))
function H = solveLinear(F,X,fid)
global EXPL
K=length(F);
% obtain coefficient mayrix of LSs
[A,b] = equationsToMatrix(F,X); tmat=toc;

% print parametric LS to solve in MUSCOD
fprintf(fid,'\nParametric linear system A*X=b\n');
for i=1:K % column-wise (FORTRAN) matrix storage
    for j=1:K
        S=['A[',num2str((j-1)+K*(i-1)),'] = ',char(vpa(A(j,i),16)),';'];
        fprintf(fid,'%s ',S);
    end
    fprintf(fid,'\n');
end
for i=1:K
    S=['b[',num2str(i-1),'] = ',char(vpa(b(i),16)),';'];
    fprintf(fid,'%s\n',S);
end

end

