function [mom,momnorm] = pce_moment(N,I,s,ext,b)
%% Returns raw and normalized moments of PC surrogate
%   [mom,momnorm] = pce_moment(N,I,s,ext,b)
%
% Input:    N           - order of moments (i.e. N+1 moments in total)
%           I           - index column vector (TK*ND) 
%           s,ext,b     - basis type of PCE-surr.
%
% empirical moments (given as argument)
% Given: N moments (moments up to order N-1) -> K=floor(N/2)+1 polynomials
% up to order floor(N/2) (if N=21 (20th order moment), K=11 (10th order))
% need last moment (even) only for normalization
% Given: K polynomials -> need N=2*(K-1)moments (+1 for normalization)

global MOMENTPATH

tname = ['K',num2str(max(I(:,1))),'_ND',num2str(length(s)),'s',num2str(s(1)),ext,'_N',num2str(N)];

% check if pce moments already exist
name = ['M',tname,'.mat'];
if exist([MOMENTPATH,name], 'file')
    disp(['Load PCE moments ',name]); load([MOMENTPATH,name]); return
end
disp(['Compute MOMENTS for PCE ',tname])

X = sym('x',[1 length(I(:,1))]);
assumeAlso(X,'real');
mom = sym('mom',[1 N+1]);
mom=subs(mom,symvar(mom),[1,X(1),zeros(1,N-1)]);
momc = sym('momc',[1 N+1]); 
momc=subs(momc,symvar(momc),[1,0,zeros(1,N-1)]);

[P,w,b] = basis(max(I(:,1)),s,ext,b);
A =  tensor(tname,N,I,P,w,b);


% ---- Computation of first and higer moments ----
for m=2:N
    % First m colums of rows corresponding to m-dim tensor T(:,:,1,..1)
    %   (by setting the last m+1 columns to 1)
    G=A(all(bsxfun(@eq, A(:,m+1:length(A(1,:))-1), ones(1,length(A(1,:))-1-m)), 2),:);
    [nrow,~]=size(G);
    % number of nonzero indices that are relevant for mom(m)
    for i=nrow:-1:1
        % 1.) Xprod is product of X coefficients
        Xprod=1;
        for mm=1:m
            Xprod=Xprod*X(G(i,mm));
        end
        % 2.) find multiplicity of index set G(i,1:m)
        U=unique(G(i,1:m));
        counter=length(U);
        fac=1;
        for j=1:counter
            %disp(['element ',num2str(U(j)),' appears ',num2str(length(find(I(i,1:m)==U(j)))),' times in ',num2str(I(i,1:m))]);
            fac=fac*factorial(length(find(G(i,1:m)==U(j))));
        end
        mult=factorial(m)/fac;
        %3.)  multiply with constant obtained from basis tensor
        mom(m+1) = mom(m+1) + mult * G(i,end) * Xprod;
        %4.) centered moments (simplified PCE form)
        if isempty(find(G(i,1:m)==1, 1))
            momc(m+1) = momc(m+1) + mult * G(i,end) * Xprod;
        end
    end
end
M=ones(1,length(mom))./sqrt(momc(3)).^(0:length(mom)-1);
momnorm=momc*diag(M); % normalized moments (of Xnorm)

save(fullfile(pwd,MOMENTPATH,name),'mom','momc','momnorm'); 


