function [] = plot_momentevol(names,TKs,s,ext,b,IND,MIND)
%% Plot evolution of state moment evolution and control evolution
%   [] = plot_momentevol(names,TKs,s,ext,b,IND,MIND)
%
% Input:    name        - solution file (.txt or .m) from solved PCE-surrogate
%           TK          - length (total order) of PCE-surr.
%           s,ext,b     - basis type of PCE-surr.
%           IND         - variable indices to plot
%           MND         - moment indices (0 - control evol., 1,.. - moment
%                         evol)
%
% Ex: plot_momentevol('ccrane_nom',10,1,'',[],6,0:2)


%% initial settings
if ~exist('IND','var'), IND=1:3; end
if ~exist('MIND','var'), MIND=[0]; end % moment indices for plotting
if ~exist('ext','var'), ext=''; end
STEP=10;

%% initialize dimensions
cellname=cellstr(names);
%if exist('exts','var'), cellext=cellstr(exts); end

%% compute PCE moments
if MIND(end)>1
    MOM = [sym('x1')*ones(length(cellname),1),zeros(length(cellname),MIND(end)-1)];
    for k=1:min(length(cellname),length(TKs))
        TK=TKs(k);
        if TK<2, continue; end
        I=index_basis(length(s),TK);
        I=I(1:TK,:);
        %if exist('exts','var'), ext=cellext{k}; end
        [mom,momnorm] = pce_moment(MIND(end),I,s,ext,b); % same parameters as in PCE
        MOM(k,:) = [mom(2),mom(3)-mom(2)*mom(2),momnorm(4:end)]; % mean, variance, 3rd to Nth normalized moments
    end
end


%% plot controls
if MIND(1)==0
    MIND=MIND(2:end);
    figure('Units','inches','Position',[0 0 4 2],'PaperPositionMode','auto');
    for k=1:min(length(cellname),length(TKs))
        name=cellname{k};
        [~,U,~] = read_solution([name,'.txt']);
        [Ngrid,Nu]=size(U);
        for p=1:Nu
            subplot(1,Nu,p,'LineWidth',3);
            stairs(0:1/(Ngrid-1):1,U(:,p)); hold on
            set(gca,'LineWidth',.1,'XTick',0:0.25:1,'XTickLabel',{'0',' ','0.5',' ','1'})
            ylabel(['$U_',num2str(p),'(t)$'])
            xlabel('$t$')
        end
    end
    %print([name(1:end-4),'_U'],'-depsc');
end

%% plot mean trajectories
if ~isempty(MIND) && MIND(1)==1
    MIND=MIND(2:end);
    figure('Units','inches','Position',[0 0 4 2],'PaperPositionMode','auto');
    for k=1:min(length(cellname),length(TKs))
        name=cellname{k};
        TK=TKs(k);
        [SD,~,~] = read_solution([name,'.txt']);
        [Ngrid,NX]=size(SD);
        NXD=floor(NX/TK);
        if ~exist([name,'.m'],'file')
            T=0:1/(Ngrid-1):1;
            II=IND;
            for j=2:TK
                II=[II,IND+(j-1)*NXD]; %II=[II,IND+1+(j-1)*NXD];  ??? 
            end
            SD = SD(:,II);
        else
            T = read_traj([name,'.m'],[1,IND(1)+1,NXD+2:NXD+TK]); % 3,12:30
            SD=T([1:STEP:end,end],2:end);
            T=T([1:STEP:end,end],1);
            [Ngrid,TK]=size(SD);
        end
        
        for i=1:length(IND)
            subplot(1,length(IND),i,'LineWidth',3); hold on
            plot(T,SD(:,i));
        end
    end
    set(gca,'LineWidth',.1,'XTick',0:0.25:1,'XTickLabel',{'0',' ','0.5',' ','1'},'Position',[.15 .2 .75 .7]);
    ylabel(['$M_1','(t)$']);
    xlabel('$t$');
    %print([name,'_M1'],'-depsc');
end

%% plot higher moments
for m=1:length(MIND)
    figure('Units','inches','Position',[0 0 4 2],'PaperPositionMode','auto');
    hold on
    for k=1:min(length(cellname),length(TKs))
        name=cellname{k};
        TK=TKs(k);
        if TK<2, continue; end
        [SD,~,~] = read_solution([name,'.txt']);
        [Ngrid,NX]=size(SD);
        NXD=floor(NX/TK);
        if ~exist([name,'.m'],'file')
            T=0:1/(Ngrid-1):1;
            II=IND;
            for j=2:TK
                II=[II,IND+(j-1)*NXD];
            end
            SD = SD(:,II);
        else
            T = read_traj([name,'.m'],[1,IND(1)+1,NXD+2:NXD+TK]); 
            SD=T([1:STEP:end,end],2:end);
            T=T([1:STEP:end,end],1);
            [Ngrid,TK]=size(SD);
        end
        X=sort(symvar(MOM(k,:)));
        tmom=zeros(1,Ngrid);
        for t=min(2,MIND(m)):Ngrid
            tmom(t)=double(subs(MOM(k,MIND(m)),X(1:TK),SD(t,:))); % normalized moment
        end
        
        %subplot(1,length(MIND),m,'LineWidth',3); hold on
        plot(T,tmom);
    end
    hold off
    
    set(gca,'LineWidth',.1,'XTick',0:0.25:1,'XTickLabel',{'0',' ','0.5',' ','1'})
    ylabel(['$M_',num2str(MIND(m))','(x_',num2str(IND(1)),'(t))$'])
    xlabel('$t$')
    
    %print([name,'_M',num2str(MIND(m))],'-depsc');
end
