function [] = sample_states(name,I,s,ext,b,IND,MC,B)
%% Sample state trajectories given solution of PCE coefficients
%   [] = sample_states(name,I,s,ext,b,IND,MC,B)
%
%   Input:    name        - solution struct from solved PCE-surrogate
%             I,s,ext,b   - index column vector (TK*ND), basis type of PCE-surr.
%             IND         - variable indices to plot
%            MC,B        - # and range of MC samples
%
%
%   Example run: sample_states('ccrane_nom',[0:9]',1,'',[],[1,6],1000)

if ~exist('B','var'), B=ones(2,1)*[-inf,inf]; end % MC sampling range
if ~exist('MC','var'), MC = 1000; end % # of MC samples
if ~exist('IND','var'), IND=1; end % indices of variables to plot

%% extract required pce coefficient solution
[SD,~,~] = read_solution([name,'.txt']);
[Ngrid,NX]=size(SD);
NXD=floor(NX/length(I(:,1)));
II=IND;
for j=2:length(I(:,1))
    II=[II,IND+(j-1)*NXD];
end
SD = SD(:,II);
T=0:1/(Ngrid-1):1;

%% build pce surrogate
[S] = pce_build(SD,I,s,ext,b);

%% generate random sample
Q = zeros(MC,length(s));
for d=1:length(s)
    if s(d) == 2
        %C(1)=sqrt(3); % 100%-CI for UD
        Q(:,d) = random('unif',-1, 1, MC,1);
    else
        Q(:,d) = random('norm',0, 1, MC,1);
        Qind=find(Q(:,d)<B(d,1)|Q(:,d)>B(d,2));
        for i=1:length(Qind)
            q=random('norm',0, 1);
            while q<B(d,1) || q>B(d,2), q=random('norm',0, 1); end
            Q(Qind(i),d)=q;
        end
    end
end


%% substitute sample in pce approximation
Xsample = zeros(MC, length(T), length(IND));
for m=1:MC
    Xsample(m,:,:) = subs(S,symvar(S),squeeze(Q(m,:)));
end


%% plot trajectories
cmap=colormap(parula);
c1=cmap(1,:); c2=cmap(floor(length(cmap)/2)-5,:); c3=cmap(end-10,:);
for i = 1:length(IND)
    subplot(1,length(IND),i,'LineWidth',3);
    title(['$x_',num2str(IND(i)-1),'(t)$'])
    xlabel('$t$')
    hold on;
    
    z = zeros(size(T));
    for m=1:MC % plot sampled trajectories
        y=Xsample(m,:,i);
        surface([T;T],[y;y],[z;z],'facecol','no','edgecol','interp','linew',0.1,'edgealpha',.25,'edgecolor',c1);
    end
    plot(T,SD(:,i),'-','LineWidth',2,'color',c3); % plot nominal trajectory
    hold off;
end
end
